-- Filename:     beeper.vhd
-- Filetype:     VHDL code (behavioral)
-- Date:         9 july 2014
-- Update:       -
-- Description:  Description of a kitchen timer beeper
-- Author:       J. op den Brouw
-- State:        demo
-- Error:        -
-- Version:      1.0
-- Copyright:    (c)2014, De Haagse Hogeschool

--
-- The sound pattern of the beeper is as followes:
-- One second is split into 16 equal parts. Every other part a beep
-- sound is produced so there are a maximum of 8 beeps per second.
-- Only the first 5 beeps are audible, the others are muted.
-- The beep frequency should be around 2 KHz, which is good for most
-- piezo beepers.

library ieee;
use ieee.std_logic_1164.all;
use ieee.numeric_std.all;

entity beeper is
    generic (freq_sys  : positive := 50000000;
             freq_beep : positive := 2000);
    port    (clk    : in std_logic;
             areset : in std_logic;
             beep   : in std_logic;
             piezo  : out std_logic
            );
end entity beeper;

architecture rtl of beeper is
constant nr_pulses : integer := freq_sys/ freq_beep / 2; -- 50M/2K/2= 12500
constant nr_nr_pulses : integer := freq_beep / 8; -- * 2 / 16 = 250
signal counter : integer range 0 to 15;
signal count_nr_pulse : integer range 0 to nr_pulses-1;
signal count_nr_nr_pulse : integer range 0 to nr_nr_pulses-1;
signal piezo_int : std_logic;
signal beep_s, beep_ss : std_logic;
begin

    process (clk, areset, beep) is
    begin
        if areset = '1' then
            counter <= 0;
            count_nr_pulse <= 0;
            count_nr_nr_pulse <= 0;
            piezo_int <= '0';
            beep_s <= '0';
            beep_ss <= '0';
        elsif rising_edge(clk) then
            beep_s <= beep;  -- synchronize
            beep_ss <= beep_s;
            if beep_ss = '1' then
                if count_nr_pulse = nr_pulses-1 then  -- 12500
                    count_nr_pulse <= 0;
                    if count_nr_nr_pulse = nr_nr_pulses-1 then -- 250
                        count_nr_nr_pulse <= 0;
                        if counter = 15 then
								    counter <= 0;
								else
									counter <= counter + 1;
								end if;
                    else
                        count_nr_nr_pulse <= count_nr_nr_pulse + 1;
                    end if;
                    if counter < 10 then
                        if (counter mod 2) = 0 then
                            piezo_int <= not piezo_int;
                        else
                            piezo_int <= '0';
                        end if;
                    else
                        piezo_int <= '0';
                    end if;
                else
                    count_nr_pulse <= count_nr_pulse + 1;
                end if;
            else
                counter <= 0;
                count_nr_pulse <= 0;
                piezo_int <= '0';
                count_nr_nr_pulse <= 0;
            end if;
        end if;
    end process;

    piezo <= piezo_int;
 
end architecture rtl;