-- Filename:     pres_beep.vhd
-- Filetype:     VHDL code (structural)
-- Date:         9 july 2014
-- Update:       -
-- Description:  PLL as prescaler with beeper
-- Author:       J. op den Brouw
-- State:        demo
-- Error:        -
-- Version:      1.0
-- Copyright:    (c)2014, De Haagse Hogeschool

-- This VHDL code demonstrates the structural description of a
-- system consisting of a Phase Locked Loop (PLL) as prescaler
-- and een beeper module that creates a well known beep sound
-- as found on digital kitchen or egg timers.
--
-- This code works directly on the Terasic DE0 board.

-- Libraries et al.
library ieee;
use ieee.std_logic_1164.all;

-- The top level entity
entity pres_beep is
		port (clk_50MHz : in std_logic;   -- pin g21  CLOCK_50
		      areset    : in std_logic;   -- pin d2   SW9
				beep      : in std_logic;   -- pin j6   SW0
				clk_32768 : out std_logic;  -- pin w13  GPIO0_D19
				locked    : out std_logic;  -- pin j1   LEDG0
				piezo     : out std_logic   -- pin ab14 GPIO0_D5
			  );
end entity pres_beep;

architecture structural of pres_beep is
-- Component description of PLL
component prescaler32768 is
	port (areset : IN STD_LOGIC  := '0';
		   inclk0 : IN STD_LOGIC  := '0';
		   c0		 : OUT STD_LOGIC;
		   locked : OUT STD_LOGIC 
	     );
end component prescaler32768;
-- Component description of the beeper
component beeper is
    generic (freq_sys  : positive := 50000000;
             freq_beep : positive := 2000);
    port    (clk    : in std_logic;
             areset : in std_logic;
             beep   : in std_logic;
             piezo  : out std_logic
            );
end component beeper;

-- Internal glue signal to glue the PLL's output clock to the beeper's input clock.
signal int_clk_32768 : std_logic;

begin

	-- The prescaler
	u0: prescaler32768
	port map (areset => areset, inclk0 => clk_50MHz, c0 => int_clk_32768, locked => locked);

	-- The beeper with input freq 32768 Hz and output (beeper) freq 2048 Hz.
   -- note that both are a power of 2.
   u1: beeper
	generic map (freq_sys => 32768, freq_beep => 2048)
	port map (clk => int_clk_32768, areset => areset, beep => beep, piezo => piezo);
	
	-- Output the internal 32768 Hz clock
	clk_32768 <= int_clk_32768;

end architecture structural;